package net.davidc.egp.snaxe.board;

import net.davidc.egp.snaxe.Direction;
import org.newdawn.slick.Color;

/**
 * @author David Croft (david@davidc.net)
 * @version $Id: Snake.java 26 2011-09-18 16:24:33Z david $
 */
public class Snake
{
  protected SnakePart head;
  protected SnakePart tail;
  protected int grow; // number of pieces to grow when we move
  protected Direction facing;

  protected Color color;
  protected boolean dead;
  protected int score;
  protected int invulnerabilityTimeRemaining;
  protected int speedMovesRemaining;

  public Snake(int x, int y, Direction facing, Color color)
  {
    head = tail = new SnakePart(this, x, y);
    this.facing = facing;
    this.color = color;
    grow = 1;
  }

  public void setFacing(Direction facing)
  {
    // Check it's not the direction we just came from
    if (head.getNextPieceDir() != facing) {
      this.facing = facing;
    }
  }

  public void move()
  {
    int newX = head.getX();
    int newY = head.getY();
    switch (facing) {
      case NORTH:
        newY--;
        break;
      case EAST:
        newX++;
        break;
      case SOUTH:
        newY++;
        break;
      case WEST:
        newX--;
        break;
    }

    SnakePart newPart = new SnakePart(this, newX, newY);
    newPart.setNext(head);
    newPart.setNextPieceDir(facing.opposite());
    head.setPrevious(newPart);
    head.setPrevPieceDir(facing);
    head = newPart;

    // If we're not growing, we need to kill the tail

    if (grow > 0) {
      grow--;
    }
    else {
      SnakePart newTail = tail.getPrevious();
      newTail.setNextPieceDir(null);
      newTail.setNext(null);
      tail = newTail;
    }
  }

  public SnakePart getHead()
  {
    return head;
  }

  public boolean isDead()
  {
    return dead;
  }

  public void setDead(boolean dead)
  {
    this.dead = dead;
  }

  public Color getColor()
  {
    return color;
  }

  public int getScore()
  {
    return score;
  }

  public void addScore(int add)
  {
    score += add;
  }

  public int getInvulnerabilityTimeRemaining()
  {
    return invulnerabilityTimeRemaining;
  }

  public boolean isInvulnerable()
  {
    return invulnerabilityTimeRemaining > 0;
  }

  public void setInvulnerabilityTimeRemaining(int invulnerabilityTimeRemaining)
  {
    this.invulnerabilityTimeRemaining = invulnerabilityTimeRemaining;
  }

  public int getSpeedMovesRemaining()
  {
    return speedMovesRemaining;
  }

  public boolean isSpeedy()
  {
    return speedMovesRemaining > 0;
  }

  public void setSpeedMovesRemaining(int speedMovesRemaining)
  {
    this.speedMovesRemaining = speedMovesRemaining;
  }

  public void grow(int pieces)
  {
    grow += pieces;
  }
}
